'use strict';

const fetchres = require('fetchres');
const logger = require('@financial-times/n-logger').default;
const elasticAgent = require('./utils/elastic-agent');
const signedFetch = require('signed-aws-es-fetch');

module.exports = function (opts) {
	const uuid = [].concat(opts.uuid);
	const returnMany = Array.isArray(opts.uuid);
	const source = 'source' in opts ? opts.source : true;
	const agent = 'agent' in opts ? opts.agent : elasticAgent;

	if (uuid.length === 0) {
		return Promise.resolve([]);
	}

	const query = {
		docs: uuid.map(function (uuid) {
			return { _id: uuid, _source: source };
		})
	};

	return signedFetch('https://next-elastic.ft.com/v3_api_v2/item/_mget', {
			method: 'POST',
			body: JSON.stringify(query),
			timeout: 3000,
			agent
		})
		.then(function (response) {
			if (!response.ok) {
				logger.warn('Failed getting ElasticSearch content', {
					uuid: uuid,
					status: response.status
				});
			}
			return response;
		})
		.then(fetchres.json)
		.then(function (data) {
			let foundCount = 0;
			const docs = data.docs

				// Filter out any that haven't been found
				.filter(function (item) {
					if (item.found) {
						foundCount++;
						return true;
					}
				})
				.map(function (item) {
					return item._source;
				});
			if (returnMany) {
				return docs;
			}
			if (foundCount === 0) {
				throw new fetchres.BadServerResponseError(404);
			}
			return docs[0];
		});
};
