'use strict';

const eagerFetch = require('./utils/eager-fetch');
const fetchres = require('fetchres');
const logger = require('@financial-times/n-logger').default;

module.exports = function (opts) {
	const uuid = opts.uuid;

	return eagerFetch('http://api.ft.com/site/v1/pages/' + uuid + '/main-content', {
			timeout: 3000,
			headers: {
				'X-Api-Key': process.env.apikey
			}
		})
			.then(function (response) {
				if (!response.ok) {
					logger.warn('Failed getting CAPIv1 page response', {
						uuid: uuid,
						type: 'Article',
						status: response.status
					});
				}
				return response;
			})
			.then(fetchres.json)
			.then(function (page) {
				const items = page.pageItems.map(function (article) {
					return article.id;
				});
				items.title = page.page.title;
				return items;
			});
};
