'use strict';

const fetchres = require('fetchres');
const logger = require('@financial-times/n-logger').default;
const eagerFetch = require('./eager-fetch');
const apiFtComAgent = require('./api-ft-com-agent');

module.exports = function (opts) {
	const uuid = opts.uuid;
	const path = opts.path;
	const type = opts.type;
	const retry = opts.retry;
	const agent = 'agent' in opts ? opts.agent : apiFtComAgent;

	return eagerFetch('http://api.ft.com' + path, {
			timeout: 3000,
			headers: {
				'X-Api-Key': process.env.api2key
			},
			retry: retry,
			agent
		})
			.then(function (response) {
				if (!response.ok) {
					logger.warn('Failed getting CAPIv2 response', {
						uuid: uuid,
						type: type,
						status: response.status
					});
				}
				return response;
			})
			.then(fetchres.json);
};
