'use strict';

const catchNetworkErrors = require('./catch-network-errors');
const logger = require('@financial-times/n-logger').default;
const BlackHoleStream = require('black-hole-stream');

module.exports = function (url, opts) {
	const retriesLeft = opts.retry === undefined ? 3 : opts.retry;
	opts.retry = undefined;
	opts.keepAlive = true;

	function fetchAttempt () {
		return fetch(url, opts)
			.catch(catchNetworkErrors)
			.then(function (response) {
				if (!response.ok && retriesLeft > 0) {
					response.body.pipe(new BlackHoleStream());
					logger.info('Failed to fetch ' + url + ' but have ' + retriesLeft + ' remaining so retrying');
					retriesLeft--;
					return fetchAttempt();
				}
				return response;
			});
	}

	return fetchAttempt();
};
